from prorab.server import send_all
from prorab import log


def task_pulled(message):
    log.log('[QUEUE] Command {} pulled from queue', message)


@send_all
def task_started(c, m):
    log.log('[EXEC] Started executing command {}', m)
    c.sendJson({'type': 'event',
        'event': 'task-started',
        'name': m, })


@send_all
def task_finished(c, p):
    log.log('[EXEC] Executing finished, returned code {}', p)
    c.sendJson({'type': 'event',
        'event': 'task-finished',
        'code': p, })


@send_all
def task_not_found(c, name):
    log.error('[EXEC] Task {} not found on local commands database!', name)
    c.sendJson({'type': 'error',
        'message': 'task {} notfound'.format(name), })


@send_all
def task_failed(c, exception):
    log.error('[EXEC] Task execution failed with exception "{}"', exception)
    c.sendJson({'type': 'error',
        'message': 'exception while running task: ' + exception, })

@send_all
def task_interrupted(c, signal, user):
    log.error('[EXEC] Task interrupted by signal {} by user {}', signal, user)
    c.sendJson({'type': 'event',
            'event': 'task-interrupted',
            'signal': signal, })



from datetime import datetime

__log = []
LOG_MAX = 1000
LOG_VIEW = 100


def _log(level, message, *args, **kwargs):
    print('[{:%y-%m-%d %H:%I:%S}] {:5.5}| {}'.format(datetime.now(), level, message.format(*args, **kwargs)))
    __log.append({
        'date': '{:%y-%m-%d %H:%I:%S}'.format(datetime.now()),
        'level': level,
        'message': message.format(*args, **kwargs),
    })

    if len(__log) > LOG_MAX:
        del __log[0]


def info(msg, *args, **kwargs):
    _log('info', msg, *args, **kwargs)


def error(msg, *args, **kwargs):
    _log('error', msg, *args, **kwargs)


def log(msg, *args, **kwargs):
    _log('log', msg, *args, **kwargs)


def view(c):
    return {'type': 'log',
            'log': __log[-LOG_VIEW:], }
